import { isEmpty } from 'lodash';
import { loadMicroApp } from './qiankun';
import { registerMicroApps } from './qiankun/apis';
import { registry_envs } from './registry/environment-entries';
import { __MICROAPP_REGISTRY } from './registry/microapp-registry';
/**
 *@readonly do not update directly use setAppsObject() in stead
 */
let asmaRegistrableApps;
export function getAsmaRegistrableApps() {
    return asmaRegistrableApps;
}
export async function setAsmaRegistrableAppsnew(reg_app_names, env = 'cloud', devtools = false) {
    const picked_microapps = reg_app_names.reduce((acc, name) => {
        acc[name] = __MICROAPP_REGISTRY[name];
        acc[name].entry = registry_envs[env][name];
        return acc;
    }, {});
    return setAsmaRegistrableApps(picked_microapps, devtools);
}
/**
 * In dev mode it  set ImportMapOverrides app
 *
 * call this function before render App() method
 * @deprecated uss set setAsmaRegistrableApps2
 * @param registrable_apps IAsmaAppsObject
 * @param devtools bool
 */
export async function setAsmaRegistrableApps(registrable_apps, devtools) {
    if (!registrable_apps) {
        devtools && console.warn('No registrable apps detected!');
        return;
    }
    if (devtools) {
        const element = document.createElement('import-map-overrides-full');
        element.setAttribute('show-when-local-storage', 'asma-devtools');
        localStorage.setItem('asma-devtools', 'true');
        document.body.appendChild(element);
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        const _ = await import('qiankun-overrides');
        const imo = window.importMapOverrides;
        imo === null || imo === void 0 ? void 0 : imo.setDefaultMap(Object.values(registrable_apps));
        asmaRegistrableApps = imo === null || imo === void 0 ? void 0 : imo.getCurrentQiankunMap();
    }
    else {
        localStorage.removeItem('asma-devtools');
        asmaRegistrableApps = registrable_apps;
    }
    if (!window.__ASMA__QIANKUN__SHELL__) {
        window.__ASMA__QIANKUN__SHELL__ = {};
    }
    window.__ASMA__QIANKUN__SHELL__.loadMicroApp = loadMicroApp;
}
/**
 * before registerAsmaMicroApps one need to call setAsmaRegistrableApps to ensure microappregistration!
 * call this method after render App() method.
 */
export async function registerAsmaMicroApps() {
    if (asmaRegistrableApps && !isEmpty(asmaRegistrableApps)) {
        registerMicroApps(Object.values(asmaRegistrableApps).filter((app) => app.activeRule !== 'component-only'), {
            beforeLoad: async (app) => {
                console.log('[LifeCycle] before load', 'color: green;', app.name);
            },
            beforeMount: async (app) => {
                console.log('[LifeCycle] before mount', 'color: green;', app.name);
            },
            afterUnmount: async (app) => {
                console.log('[LifeCycle] after unmount', 'color: green;', app.name);
            },
        });
    }
}
//# sourceMappingURL=registerASMAMicroApps.js.map