/* eslint-disable @typescript-eslint/ban-ts-comment */
/* eslint-disable @typescript-eslint/no-explicit-any */
/**
 * @author Hydrogen
 * @since 2020-3-8
 */
import type { SandBox } from '../interfaces'
import { SandBoxType } from '../interfaces'

function iter(obj: typeof window, callbackFn: (prop: any) => void) {
    // eslint-disable-next-line guard-for-in, no-restricted-syntax
    for (const prop in obj) {
        // patch for clearInterval for compatible reason, see #1490
        if (obj.hasOwnProperty(prop) || prop === 'clearInterval') {
            callbackFn(prop)
        }
    }
}

/**
 * 基于 diff 方式实现的沙箱，用于不支持 Proxy 的低版本浏览器
 */
export default class SnapshotSandbox implements SandBox {
    proxy: WindowProxy

    name: string

    type: SandBoxType

    sandboxRunning = true

    private windowSnapshot!: Window

    private modifyPropsMap: Record<any, any> = {}

    constructor(name: string) {
        this.name = name
        this.proxy = window
        this.type = SandBoxType.Snapshot
    }

    active() {
        // 记录当前快照
        this.windowSnapshot = {} as Window
        iter(window, (prop) => {
            //@ts-ignore
            this.windowSnapshot[prop] = window[prop]
        })

        // 恢复之前的变更
        Object.keys(this.modifyPropsMap).forEach((p: any) => {
            window[p] = this.modifyPropsMap[p]
        })

        this.sandboxRunning = true
    }

    inactive() {
        this.modifyPropsMap = {}

        iter(window, (prop) => {
            if (window[prop] !== this.windowSnapshot[prop]) {
                // 记录变更，恢复环境
                this.modifyPropsMap[prop] = window[prop]
                //@ts-ignore
                window[prop] = this.windowSnapshot[prop]
            }
        })
        //@ts-ignore
        if (process.env.NODE_ENV === 'development') {
            console.info(`[qiankun:sandbox] ${this.name} origin window restore...`, Object.keys(this.modifyPropsMap))
        }

        this.sandboxRunning = false
    }
}
